/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <project.h>
#include <math.h>


typedef enum {      //enumerate modes of operation
    TILT,
    ACCELERATION,
    SPIN
} mode_t;

typedef struct MPU_DATA {       //a structure to hold the measurements from the MPU
    float x_gyro;
    float y_gyro;
    float z_gyro;
    
    float x_accel;
    float y_accel;
    float z_accel;
} MPU_DATA_t;


mode_t mode = TILT;             //start mode is TILT
MPU_DATA_t mpu_data;            //struct to hold our MPU data


void tilt ();                   //mode function prototypes
void acceleration();
void spin ();

void read_MPU();                //helper function prototypes
void MPU_ByteWrite(uint8_t reg, uint8_t byte);
void switch_modes();


int main()
{
    CyGlobalIntEnable;          //Enable Global Interrupts
    
    LED_CONTROL_Write(0x0);     //initialize the LED control register to zero
    CyDelay(50);
    
    LED_CONTROL_Write(0x3F);    //write all LEDs on
    CyDelay(500);
    
    LED_CONTROL_Write(0x2A);    //write even LEDs on
    CyDelay(500);
    
    LED_CONTROL_Write(0x15);    //write odd LEDs on
    CyDelay(500);
    
    LED_CONTROL_Write(0x0);     //write all LEDs off
    
    I2C_MASTER_1_Start();       //Start the I2C Master
    
    I2C_MASTER_1_MasterSendStart(0x68, 0);      //starts an I2C write. The address of the MPU is 0x68
    I2C_MASTER_1_MasterWriteByte(0x1A);         //writes the address of the target register. 1A is the CONFIG register
    I2C_MASTER_1_MasterWriteByte(0x06);         //writes sets the frame sync to disabled and the DLPF to a bandwidth of 260 Hz
    I2C_MASTER_1_MasterSendStop();              //stops an I2C transaction.
    

    MPU_ByteWrite(0x6B, 0x00);  //takes the MPU out of sleep mode
    MPU_ByteWrite(0x1B, 0x00);  //sets the gyros to 250 degrees/second range
    MPU_ByteWrite(0x1C, 0x00);  //sets the accelerometer to +/- 2 gees range
    MPU_ByteWrite(0x23, 0x00);  //disables FIFO buffer
    
    
    for(;;)
    {
        
        read_MPU();                 //get the MPU data
        switch_modes();             //switch the mode variable as appropriate
        switch(mode){
            case TILT:              //if mode == TILT, do tilt
                tilt();
                break;
            case ACCELERATION:      //if mode == ACCELERATION, do acceleration
                acceleration();
                break; 
            case SPIN:              //if mode == SPIN, do spin
                spin();
                break;
        }
    }
}

void tilt(){
    if (mpu_data.x_accel >= 0.25)  LED_CONTROL_Write(LED_CONTROL_Read() | 0x1);     //if the x axis acceleration is greater than .25 gee, turn on the far right LED
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xFE);                              //otherwise, turn it off
    if (mpu_data.x_accel <= -0.25) LED_CONTROL_Write(LED_CONTROL_Read() | 0x2);     //if the x is less than .25 gee, turn the second from the right on
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xFD);                              //otherwise, turn it off
    
    if (mpu_data.y_accel >= 0.25) LED_CONTROL_Write(LED_CONTROL_Read() | 0x4);      //repeat for the y and z axes and their respective LEDs
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xFB);
    if (mpu_data.y_accel <= -0.25) LED_CONTROL_Write(LED_CONTROL_Read() | 0x8);
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xF7);
    
    if (mpu_data.z_accel >= 0.25) LED_CONTROL_Write(LED_CONTROL_Read() | 0x10);
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xEF);
    if (mpu_data.z_accel <= -0.25) LED_CONTROL_Write(LED_CONTROL_Read() | 0x20);
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xDF);
    
}


void acceleration(){
    float total_accel = sqrtf(mpu_data.x_accel * mpu_data.x_accel + mpu_data.y_accel * mpu_data.y_accel + mpu_data.z_accel * mpu_data.z_accel); //take the magnitude of the total acceleration
    LED_CONTROL_Write((uint8_t)(total_accel/2 * 64));   //write the 6-bit representation to the LED bank
}

void spin(){
    if (mpu_data.x_gyro >= 50.0) LED_CONTROL_Write(LED_CONTROL_Read() | 0x1);   //this is the same as tilt() but it uses the gyros with a threshold of 50 deg/s
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xFE);
    if (mpu_data.x_gyro <= -50.0) LED_CONTROL_Write(LED_CONTROL_Read() | 0x2);
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xFD);
    
    if (mpu_data.y_gyro >= 50.0) LED_CONTROL_Write(LED_CONTROL_Read() | 0x4);
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xFB);
    if (mpu_data.y_gyro <= -50.0) LED_CONTROL_Write(LED_CONTROL_Read() | 0x8);
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xF7);
    
    if (mpu_data.z_gyro >= 50.0) LED_CONTROL_Write(LED_CONTROL_Read() | 0x10);
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xEF);
    if (mpu_data.z_gyro <= -50.0) LED_CONTROL_Write(LED_CONTROL_Read() | 0x20);
    else LED_CONTROL_Write(LED_CONTROL_Read() & 0xDF);
    
}

void switch_modes (){
   if (!SW_1_Read() && mode == TILT) mode = SPIN;               //switch is low when pressed. if it is pressed, cycle to the next mode
   else if(!SW_1_Read() && mode == SPIN) mode = ACCELERATION;
   else if (!SW_1_Read() && mode == ACCELERATION) mode = TILT;
   else return;
   CyDelay(250);    //lazy debouncing. halt the processor for .25 seconds after a mode change so it doesn't jump through a bunch
}


void read_MPU(){
    uint8_t rawdata [6] = {0};
    
    I2C_MASTER_1_MasterSendStart(0x68, 0);      //start a write transaction to send the MPU the register address to start reading from
    I2C_MASTER_1_MasterWriteByte(0x43);         //the gyro registers begin at 0x43
    I2C_MASTER_1_MasterSendRestart(0x68, 1);    //start a read transaction to begin receiving data from the MPU
                                                //Note that we must use MasterSendRestart here because we previously sent a start condition without following it with a stop condition.
    
    
    rawdata[0] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);    //read bytes 0-4 and acknowledge
    rawdata[1] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);
    rawdata[2] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);
    rawdata[3] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);
    rawdata[4] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);
   
    rawdata[5] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_NAK_DATA);    //read byte 5 and send non-acknowledge to signal end of reading data
    
    I2C_MASTER_1_MasterSendStop();              //send stop byte to finish the transaction.
    
    uint16_t hold = ((uint16_t)rawdata[0] << 8) | (uint16_t)rawdata[1]; //concatenate the upper and lower byte of each value
    mpu_data.x_gyro = (int16_t)hold / 131.0;                         //interpret each value and put it into the mpu_data structure
    
    hold = ((uint16_t)rawdata[2] << 8) | (uint16_t)rawdata[3];
    mpu_data.y_gyro = (int16_t)hold / 131.0;
        
    hold = ((uint16_t)rawdata[4] << 8) | (uint16_t)rawdata[5];
    mpu_data.z_gyro = (int16_t)hold / 131.0;
    
    
    I2C_MASTER_1_MasterSendStart(0x68, 0);      //start a write transaction to send the MPU the register address to start reading from
    I2C_MASTER_1_MasterWriteByte(0x3B);         //the accelerometer registers begin at 0x3B
    I2C_MASTER_1_MasterSendRestart(0x68, 1);      //start a read transaction to begin receiving data from the MPU
    
    rawdata[0] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);    //read bytes 0-4 and acknowledge
    rawdata[1] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);
    rawdata[2] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);
    rawdata[3] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);
    rawdata[4] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);
   
    rawdata[5] = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_NAK_DATA);    //read byte 5 and send non-acknowledge to signal end of reading data
    
    I2C_MASTER_1_MasterSendStop();              //send stop byte to finish the transaction.
    
    hold = ((uint16_t)rawdata[0] << 8) | (uint16_t)rawdata[1];
    mpu_data.x_accel = (int16_t)hold / 16384.0;  

    hold = ((uint16_t)rawdata[2] << 8) | (uint16_t)rawdata[3];
    mpu_data.y_accel = (int16_t)hold / 16384.0;
        
    hold = ((uint16_t)rawdata[4] << 8) | (uint16_t)rawdata[5];
    mpu_data.z_accel = (int16_t)hold / 16384.0;
    
}

void MPU_ByteWrite(uint8_t reg, uint8_t byte){
    I2C_MASTER_1_MasterSendStart(0x68, 0);      //start a write to the MPU to begin the transaction
    I2C_MASTER_1_MasterWriteByte(reg);          //send the address of the register we want to write to
    I2C_MASTER_1_MasterWriteByte(byte);         //send the byte
    I2C_MASTER_1_MasterSendStop();              //send the stop signal to end the transaction
}


/* [] END OF FILE */
